﻿/// <reference path="http://ajax.microsoft.com/ajax/jQuery/jquery-1.4.1-vsdoc.js"/>
// you can remove above line - it's only for intellisense in Visual Studio

//Demo code
var db;
var curVersion = 27;
var tableName = "testTable";
var tableKey = null;
var indexData = [
    {
        displayName: "Age",
        name: "AgeIndex",
        key: "age"
    },
//    {
//        displayName: "Gender",
//        name: "idbIndexByGender",
//        key: "gender"
//    },
    {
        displayName: "Name",
        name: "NameIndex",
        key: "name"
    }
];
var rec = {
    date: 0,
    name: "",
    age: 0,
    gender: ""
};

var indicies = {};

var useObjectCursorForIndex = true;

$(document).ready(function () {
    //write your jQuery code here
    $("#add").mousedown(addRandom);
    $("#count").mousedown(count);
    $("#list").mousedown(list);
    $("#reverseList").mousedown(reverseList);

    $("#indexListEntry").tmpl(indexData).appendTo("#listIndex");
    $("#indexListEntry").tmpl(indexData).appendTo("#reverseListIndex");
    $("#listIndex").change(list);
    $("#reverseListIndex").change(reverseList);

    if (window.indexedDB) {
        //IndexeDB support
        indexedDBImpl = window.indexedDB;
    } else if (window.moz_indexedDB) {
        indexedDBImpl = window.moz_indexedDB;
    } else {
        alert("No indexed db support");
    }

    if (indexedDBImpl) {
        try {
            connectionOpening(
                indexedDBImpl.open("test", "test database"),
                function (db) {
                    log("db opened");
                    list();
                },
                function (errMsg) {
                    log("db failed to open:" + errMsg);
                }
            );
        } catch (ex) {
            log("Excepion thrown while opening db:" + ex.message);
        }
    }
});

var names = ["bob", "tom", "craig", "fred", "sally", "kim", "sienna", "amy"];
var lastNames = ["smith", "jones", "carpenter", "holmes", "shank", "steel", "peters", "cooper", "chance", "peters"];
function addRandom() {
    rec.date = new Date();
    var t = rec.date.getTime();
    var nameInd = Math.floor(t / 10) % names.length;
    rec.name = names[nameInd];
    if (nameInd >= (names.length / 2)) {
        rec.gender = "female";
    } else {
        rec.gender = "male";
    }
    nameInd = Math.floor(t / 100) % lastNames.length;
    rec.name += " " + lastNames[nameInd];
    rec.age = t % 80;
    add(rec,
        function (key) {
            $("#addResult").text("Added record at key " + key);
        },
        function (err) {
            $("#addResult").text("Add failed: " + err);
        }
    );
}

function count() {
    getRowCount(
        function (c) {
            $("#countResult").text(c + " records");
        },
        function (errMsg) {
            $("#countResult").text("Count failed: " + errMsg);
        }
    );
}

function listSuccess(records) {
    $("#listDisplay *").remove();
    if (records && records.length > 0) {
        $("#recordDisplay").tmpl(records).appendTo("#listDisplay");
    } else {
        $("#listDisplay").append("<li></li>");
        $("#listDisplay li").text("No records listed...");
    }
}

function listFailure(error) {
    $("#listDisplay *").remove();
    $("#listDisplay").append("<li></li>");
    $("#listDisplay li").text("Error listing records: " + error);
}

function list() {
    var listByIndex = $("#listIndex").val();
    if (!listByIndex) {
        $("#listTitle").text("ascending list in default order");
    } else {
        $("#listTitle").text("ascending list ordered by index " + listByIndex);
    }
    get("first", "all", listSuccess, listFailure, { index: listByIndex });
}

function reverseList() {
    var listByIndex = $("#reverseListIndex").val();
    if (!listByIndex) {
        $("#listTitle").text("descending list in default order");
    } else {
        $("#listTitle").text("descending list ordered by index " + listByIndex);
    }
    get("last", "allbefore", listSuccess, listFailure, { index: listByIndex });
}

function inv(f, arg1, arg2) {
    if (f) {
        f(arg1, arg2);
    }
}

function log(msg) {
    if (window.console && window.console.log) {
        window.console.log(msg);
    } else if (window.opera) {
        window.opera.postError(msg);
    } else if (window.debugService) {
        window.debugService.trace(msg);
    }
}


//DB Handling code

function connectionOpening(connectionRequest, success, failure) {
    if (connectionRequest) {
        connectionRequest.onsuccess = function (evt) {
            //logger.info("Async connection request completed successfully.");
            connectionOpened(evt.result, success, failure);
        };
        connectionRequest.onerror = function (evt) {
            //logger.exception("Async connection request failed: ", evt);
            inv(failure, evt.message);
        };
    }
}

function connectionOpened(openedDB, success, failure) {
    db = openedDB;
    try {
        var dbVersion = db.version;
        if (!dbVersion) {
            dbVersion = "0";
        }
        dbVersion = parseInt(dbVersion);
        if (dbVersion < curVersion) {
            //We have an async db interface, you can only create the object store in the
            //success callback to the setVersion() method.
            var setVersionReq = db.setVersion(curVersion);
            setVersionReq.onsuccess = function (evt) {
                try {
                    if (dbVersion > 0) {
                        //Simple upgrade, just toss the old object store.
                        var removeReq = db.removeObjectStore(tableName);
                        removeReq.onsuccess = function (evt) {
                            finishCreate(success, failure);
                        };
                    } else {
                        finishCreate(success, failure);
                    }
                } catch (ex) {
                    alert("Hit exception trying to remove old object store: " + ex.message);
                    finishCreate(success, failure);
                }
            };
            setVersionReq.onerror = function (evt) {
                alert("Failed to set db version: "+ evt.message);
                inv(failure, evt.message);
            };
        } else {
            var objStore = getObjectStore(tableName, IDBTransaction.READ_ONLY);
            for (var indInd = 0; indInd < indexData.length; indInd++) {
                var indexInfo = indexData[indInd];
                var indOrReq = objStore.index(indexInfo.name);
                if (indOrReq) {
                    if (indOrReq.name) {
                        indicies[indOrReq.name] = indOrReq;
                    } else {
                        //I swear I saw it go though this path once.
                        log("objStore.index() returned a IDBRequest instead of an IDBIndex");
                        indOrReq.onsuccess = function (evt) {
                            if (evt.result) {
                                indicies[evt.result.name] = evt.result;
                            } else {
                                alert("index not found when re-opening DB");
                            }
                        }
                        indOrReq.onerror = function (evt) {
                            alert("index not found when re-opening DB");
                        }
                    }
                } else {
                    alert("index " + indexInfo.name + "not found when re-opening DB");
                }
            }
            inv(success);
        }
    } catch (exception) {
        inv(failure, exception.message);
    }
}

function finishCreate(success, failure) {
    var createRequest = db.createObjectStore(tableName, tableKey, true);
    createRequest.onsuccess = function (evt) {
        var objStore = evt.result;
        createIndicies(objStore, success, failure);
    };
    createRequest.onerror = function (evt) {
        alert("Failed to create object store: " + evt.message);
        inv(failed);
    }
}

function getObjectStore(name, mode) {
    return db.objectStore(name, mode);
}

function createIndicies(objStore, success, failure) {
    if (!objStore) {
        objStore = getObjectStore(tableName, IDBTransaction.READ_ONLY);
    }
    var created = 0;
    for (var indInd = 0; indInd < indexData.length; indInd++) {
        var indexInfo = indexData[indInd];
        var indReq = objStore.createIndex(indexInfo.name, indexInfo.key, false);
        indReq.onsuccess = function (evt) {
            indicies[evt.result.name] = evt.result;
            if (++created >= indexData.length) {
                inv(success);
            }
        }
        indReq.onerror = function (evt) {
            alert("Failed to create an index");
            inv(failure, "Failed to create an index");
        }
    }
}

function getRowCount(success, failure) {
    var cursorRequest = getObjectStore(tableName, IDBTransaction.READ_ONLY).openCursor();
    var count = 0;
    var lastKey;
    cursorRequest.onsuccess = function (evt) {
        var cursor = evt.result;
        if (cursor) {
            count++;
            lastKey = evt.key;
            //Just because the continue in cursor.continue(), gets painted with the keyword color.
            cursor["continue"]();
        } else {
            inv(success, count, lastKey);
        }
    }
    cursorRequest.onerror = function (evt) {
        alert("Failed to get cursor: " + evt.messag);
        inv(failure, evt.message);
    }
}

function get(firstKey, count, success, failure, options) {
    try {
        var all = false;
        var includeDups = !options || !options.noDups;
        var direction = includeDups ? IDBCursor.NEXT : IDBCursor.NEXT_NO_DUPLICATE;
        var countLower = count.toString().toLowerCase();
        if (countLower == "all" || countLower == "allafter") {
            all = true;
            count = Number.MAX_VALUE;
        } else if (countLower == "allbefore") {
            direction = includeDups ? IDBCursor.PREV : IDBCursor.PREV_NO_DUPLICATE;
            count = Number.MAX_VALUE;
            all = true;
        } else if (count < 0) {
            direction = includeDups ? IDBCursor.PREV : IDBCursor.PREV_NO_DUPLICATE;
            count *= -1;
        }

        var cursorRange;
        var firstLower = firstKey.toString().toLowerCase();
        if (firstLower == "first" || firstLower == "last") {
            cursorRange = null;
        } else {
            if (direction == IDBCursor.NEXT || direction == IDBCursor.NEXT_NO_DUPLICATE) {
                //cursorRange = IDBKeyRange.leftBound(firstKey)
                cursorRange = {
                    left: firstKey,
                    flags: IDBKeyRange.LEFT_BOUND | IDBKeyRange.RIGHT_OPEN
                };
            } else {
                //cursorRange = IDBKeyRange.rightBound(firstKey);
                cursorRange = {
                    right: firstKey,
                    flags: IDBKeyRange.LEFT_OPEN | IDBKeyRange.RIGHT_BOUND
                };
            }
        }
        var cursorRequest;
        var objStore = getObjectStore(tableName, IDBTransaction.READ_ONLY);
        var index = null;
        var usingObjectCursor = true;
        if (options && options.index) {
            if (indicies[options.index]) {
                index = indicies[options.index];
                usingObjectCursor = useObjectCursorForIndex;
                if (usingObjectCursor) {
                    cursorRequest = index.openObjectCursor(cursorRange, direction);
                } else {
                    cursorRequest = index.openCursor(cursorRange, direction);
                }
            } else {
                inv(failure, "No such index: " + options.index);
            }
        } else {
            cursorRequest = objStore.openCursor(cursorRange, direction);
        }
        var results = [];
        var cursorInd = 0;
        var resultInd = 0;
        cursorRequest.onsuccess = function (evt) {
            var cursor = evt.result;
            if (cursor) {
                cursorInd++;
                var r;
                if (usingObjectCursor) {
                    r = cursor.value;
                    r.date = new Date(r.date);
                    results[resultInd++] = r;
                } else {
                    //When not using an object cursor we need to do an additional
                    //lookup because the cursor value is a key into the index.
                    var getReq = index.get(cursor.value);
                    getReq.onsuccess = function (evt) {
                        r = evt.result;
                        r.date = new Date(r.date);
                        results[resultInd++] = r;
                        if (!all && resultInd >= count) {
                            inv(success, results);
                        }
                    }
                    getReq.onerror = function (evt) {
                        log("Shouldn't get here, the key returned by the cursor should exist");
                        inv(failure, "failed to retrieve object for cursor key");
                    }
                }
                if (all || cursorInd < count) {
                    cursor["continue"]();
                } else if(usingObjectCursor){
                    inv(success, results);
                }
            } else {
                if (all) {
                    inv(success, results);
                } else {
                    inv(failure, "Not enough items in db or index to satisfy request");
                }
            }
        };
        cursorRequest.onerror = function (evt) {
            alert("Failed to get cursor: " + evt.messag);
            inv(failure, evt.message);
        }
    } catch (exception) {
        inv(failure, "Get failed: " + exception.message);
    }
}

function add(r, success, failure) {
    var objStore = getObjectStore(tableName, IDBTransaction.READ_WRITE);
    try {
        var addReq = objStore.add(r);
        addReq.onsuccess = function (evt) {
            inv(success, evt.result);
        };
        addReq.onerror = function (evt) {
            inv(failure, evt.message);
        };
    } catch (exception) {
        inv(failure, exception.message);
        alert("Exception thrown while adding: " + exception.message);
    }
}
